/***************************************************************************
 *            Torrent.cc
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "Lince/Torrent.h"

#include "Lince/Core.h"
#include "Lince/Utils.h"
#include "Lince/AlertsManager.h"

#include "libtorrent/bencode.hpp"
#include "libtorrent/peer_id.hpp"

#include "../config.h"
#include "Lince/i18n.h"

#include <float.h>

using namespace Lince;

Torrent::Torrent():
	th(),
	timer()
{
	pos = -1;
	isQueued = isStopped = isComplete = false;
	failedHashes = 0;
	uploadsSlots = maxConnects = maxUpload = maxDownload = -1;
	shareRatio = 0.0;
	totalTime = totalUploaded = 0;
	storageMode = libtorrent::storage_mode_allocate;
}

Torrent::Torrent(const libtorrent::torrent_handle &th):
	timer()
{
	pos = -1;
	isQueued = isStopped = isComplete = false;
	failedHashes = 0;
	uploadsSlots = maxConnects = maxUpload = maxDownload = -1;
	shareRatio = 0.0;
	totalTime = totalUploaded = 0;
	storageMode = libtorrent::storage_mode_allocate;
	set_handle(th);
}

void Torrent::stop()
{
	if (isStopped)
		return;
	isStopped = true;

	Core::get_alerts_manager()->add_local_alert(local_alert(STOPPED, INFO, name(), info_hash()));

	if (!isQueued)
		th.pause();
}

void Torrent::start()
{
	if (!isStopped)
		return;
	isStopped = false;

	Core::get_alerts_manager()->add_local_alert(local_alert(STARTED, INFO, name(), info_hash()));

	if (!isQueued)
		th.resume();
}

void Torrent::queue()
{
	if (isQueued)
		return;
	isQueued = true;

	Core::get_alerts_manager()->add_local_alert(local_alert(QUEUED, INFO, name(), info_hash()));

	if (!isStopped)
		th.pause();
}

void Torrent::unqueue()
{
	if (!isQueued)
		return;
	isQueued = false;

	Core::get_alerts_manager()->add_local_alert(local_alert(UNQUEUED, INFO, name(), info_hash()));

	if (!isStopped)
		th.resume();
}

// This function needs to be called after set_file_priorities
void Torrent::prioritize_first_part_of_files(int priority, boost::int64_t minsize)
{
	int i;
	libtorrent::file_entry file_entry;
	std::vector<libtorrent::file_entry>::const_iterator file_iterator;

	// Check that we are in a valid torrent and file priorities are set.
	if (!th.is_valid() || filePriorities.empty()) return;

	libtorrent::torrent_info ti = get_torrent_info();

	for (file_iterator = ti.begin_files(), i = 0; file_iterator != ti.end_files(); ++file_iterator, i++)
	{
		file_entry = ti.file_at(i);

		// Do not prioritize files that are not going to be downloaded
		// Do not prioritize files smaller than specified size
		// Do not change files with a higher priority
		if (file_entry.size < minsize || filePriorities[i] == 0 ||
				filePriorities[i] >= priority)
			continue;

		int first = ti.map_file(i, 0, 1).piece;
		int last = ti.map_file(i, file_entry.size - 1, 1).piece;
		int pieces = (int) ((last-first) * 10 / 100);

		// Prioritize first and last parts
		for (int j=first; j<(first+pieces); j++)
				th.piece_priority(j, priority);

		for (int j=last; j>(last-pieces); j--)
				th.piece_priority(j, priority);
	}
}

libtorrent::entry Torrent::create_torrent()
{
	const char *buf = th.get_torrent_info().metadata().get();
	int data_size = th.get_torrent_info().metadata_size();

	// For creation date: get the number of seconds since January 1:st 1970.
	boost::gregorian::date d(1970,boost::gregorian::Jan,01);
	boost::posix_time::ptime p1(d, boost::posix_time::seconds(0));
	boost::posix_time::ptime p2(boost::posix_time::second_clock::local_time());

	libtorrent::entry torrent_entry;
	torrent_entry["announce"] = th.trackers()[0].url;
	torrent_entry["comment"] = th.get_torrent_info().comment();
	torrent_entry["creation date"] = boost::posix_time::time_period(p1, p2).length().total_seconds();
	torrent_entry["info"] = libtorrent::bdecode(buf, buf + data_size);

	return torrent_entry;
}

void Torrent::save_resume_data()
{
	if (!th.has_metadata() || !th.is_valid()) return;
	th.save_resume_data();
}

float Torrent::get_ratio()
{
	if (!status().total_done)
		return 0.0;

	return static_cast<float>(total_uploaded()
			/ (float) status().total_done);
}

float Torrent::get_eta()
{
	if (status().download_payload_rate)
		return ((get_torrent_info().total_size() - status().total_done)
			/ status().download_payload_rate);

	return FLT_MAX;
}

std::string Torrent::get_status_string()
{
	int state = status().state;

	if (state == libtorrent::torrent_status::checking_files)
		return std::string(_("Checking"));
	if (state == libtorrent::torrent_status::queued_for_checking)
		return std::string(_("Queued for checking"));

	if (isStopped)
		return std::string(_("Stopped"));
	if (isQueued)
		return std::string(_("Queued"));

	switch (state){
			case libtorrent::torrent_status::downloading_metadata:
				return std::string(_("Downloading metadata"));
				break;
			case libtorrent::torrent_status::downloading:
				return std::string(_("Downloading"));
				break;
			case libtorrent::torrent_status::finished:
				return std::string(_("Finished"));
				break;
			case libtorrent::torrent_status::seeding:
				return std::string(_("Seeding"));
				break;
			case libtorrent::torrent_status::allocating:
				return std::string(_("Allocating"));
				break;
			default:
				return std::string(_("Unknown"));
				break;
		}
}


// -1 -> Unknown | 0 -> None file not downloaded
// 1 -> Normal | 2 -> High | 7 -> Maximum
int Torrent::get_priority_of_file(int file_number)
{
	if (file_number < 0 || file_number > filePriorities.size())
		return -1;

	return filePriorities[file_number];
}

std::string Torrent::get_date_as_string(void)
{
	if (get_torrent_info().creation_date())
		return boost::posix_time::to_simple_string(get_torrent_info().creation_date().get());

	return "";
};
