/***************************************************************************
 *            MainWindow.cc
 *
 *  Mon Dec 18 21:15:17 2006
 *  Copyright  2006  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include <unistd.h>
#include <stdlib.h>

#include "MainWindow.h"
#include "XMLWriter.h"
#include "LoadTorrentDialog.h"
#include "EditTrackersDialog.h"

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"
#include "libtorrent/alert_types.hpp"

#include "libtorrent/extensions/ut_pex.hpp"
#include "libtorrent/extensions/metadata_transfer.hpp"

#include "../config.h"

#include "Lince/Core.h"
#include "Lince/SetupManager.h"

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>

#include <glibmm/i18n.h>

#include <iostream>
#include <vector>
#include <algorithm>
#include <sstream>

/* TODO: Decidir que hacer con el resume data
		 Añadir mutex para updateApp
 		 Hacer que el usuario pueda decidir el tiempo de actualizacion del gui
*/

using namespace Lince;

MainWindow::MainWindow():
 		vbox(false, 0),
		menuBar(this),
		systray(this),
		notifications(&systray),
		noteBookManager(this, &menuBar, &notifications)
{
		set_title("Lince");
		set_default_size(740, 540);
		set_border_width(1);
		maximize();

		vbox.pack_start(*menuBar.getMenuBar(), Gtk::PACK_SHRINK, 0);
		vbox.pack_start(*menuBar.getToolBar(), Gtk::PACK_SHRINK, 0);

		vbox.pack_start(noteBookManager, Gtk::PACK_EXPAND_WIDGET);

		statusBar.push(PACKAGE_STRING);
		vbox.pack_start(statusBar, Gtk::PACK_SHRINK, 0);

		add(vbox);
		show_all_children();

		// Signals
		signal_hide().connect( sigc::mem_fun(*this, &MainWindow::on_signal_hide) );

		makeTorrentWindow.set_transient_for(*this);

		systray.show();
		isShown = true;

		set_focus_child(noteBookManager);
		set_icon(Gdk::Pixbuf::create_from_file(Glib::build_filename(ICON_DIR, "lince.png")));

		updateGuiSetup();
		noteBookManager.setOptionsGuiSetupManager(&guiSetupManager);
		noteBookManager.setSearchSetupManager(&guiSetupManager);

		updateGui();

		// Update Gui
		Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::updateGui), 1000);
}

// Destructor
MainWindow::~MainWindow()
{
	std::cout << _("Exiting") << std::endl;

	// Update Gui Setup
	guiSetupManager.set(GuiSetupManager::SHOW_TORRENTSTAB, noteBookManager.isTorrentsTabShown());
	guiSetupManager.set(GuiSetupManager::SHOW_OPTIONSTAB, noteBookManager.isOptionsTabShown());
	guiSetupManager.set(GuiSetupManager::SHOW_STATISTICSTAB, noteBookManager.isStatisticsTabShown());
	guiSetupManager.set(GuiSetupManager::SHOW_LOGTAB, noteBookManager.isLogTabShown());
	guiSetupManager.set(GuiSetupManager::SHOW_TOOLBAR, menuBar.isToolBarShown());
	guiSetupManager.set(GuiSetupManager::SHOW_SEARCHTAB, noteBookManager.isSearchTabShown());
	guiSetupManager.save();
}

bool MainWindow::updateGui()
{
	static int updateGraphics = 0, updateFastResume=0;
	SessionManagerPtr ssm = Core::get_session_manager();
	TorrentsManagerPtr tm = Core::get_torrents_manager();
	SetupManagerPtr sm = Core::get_setup_manager();

	// Update Torrents
	noteBookManager.updateDownloadingTorrents();
	noteBookManager.updateDownloadedTorrents();

	noteBookManager.updateTorrentsInfoTab();

	// Update systray and StatusBar
	systray.setToolTip(ssm->status().upload_rate, ssm->status().download_rate,
					   tm->downloading_torrents_count(), tm->shared_torrents_count());

	statusBar.setUpSpeed(ssm->status().upload_rate);
	statusBar.setDownSpeed(ssm->status().download_rate);
	statusBar.setConnectionsNum(ssm->status().num_peers);

	statusBar.setMaxUpSpeed(ssm->upload_rate_limit());
	statusBar.setMaxDownSpeed(ssm->download_rate_limit());

	statusBar.setDhtStatus(sm->getBool(SetupManager::DHT_SUPPORT),
			ssm->status().dht_nodes, ssm->status().dht_global_nodes);

	if (sm->getBool(SetupManager::DISABLE_CONNECTIONS_NUMBER))
		statusBar.setMaxConnections(0);
	else
		statusBar.setMaxConnections(sm->get(SetupManager::CONNECTIONS_NUMBER));

	if (ssm->status().has_incoming_connections){
		statusBar.setStatus(StatusBar::GOOD);
	}
	else if (!ssm->status().has_incoming_connections && !tm->downloading_torrents_count()
			 && tm->shared_torrents_count()) {
		statusBar.setStatus(StatusBar::SEEDING);
	}
	else if (ssm->status().num_peers){
		statusBar.setStatus(StatusBar::FIREWALLED);
	}
	else
		statusBar.setStatus(StatusBar::NO_CONNECTION);

	// Update Statistics tab
	noteBookManager.updateTimeSession(ssm->get_session_elapsed_time());
	noteBookManager.updateTimeTotal(sm->get(SetupManager::TOTAL_TIME) + ssm->get_session_elapsed_time());

	noteBookManager.updateDownloadSpeed(ssm->status().download_rate);
	noteBookManager.updateDownloadSession(ssm->status().total_download);
	noteBookManager.updateDownloadTotal(sm->get(SetupManager::TOTAL_DOWNLOAD) + ssm->status().total_download);

	noteBookManager.updateUploadSpeed(ssm->status().upload_rate);
	noteBookManager.updateUploadSession(ssm->status().total_upload);
	noteBookManager.updateUploadTotal(sm->get(SetupManager::TOTAL_UPLOAD) + ssm->status().total_upload);

	noteBookManager.updateShareSession();
	noteBookManager.updateShareTotal();

	noteBookManager.updateCurrentDownloadingSize(ssm->get_current_downloading_size());
	noteBookManager.updateTotalDownloadingSize(ssm->get_total_downloading_size());
	noteBookManager.updateDownloadingRatio();
	noteBookManager.updateTotalSharedSize(ssm->get_total_shared_size());

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_GRAPHIC_STATISTICS))
	{
		noteBookManager.addCurrentDownloadSpeed(ssm->status().download_rate);
		noteBookManager.addCurrentUploadSpeed(ssm->status().upload_rate);

		if (!updateGraphics)
		{
			if(!sm->getBool(SetupManager::DISABLE_DOWNLOAD_SPEED))
				noteBookManager.setMaxDownloadSpeed(sm->get(SetupManager::DOWNLOAD_SPEED) * 1024);
			else
				noteBookManager.setMaxDownloadSpeed(ssm->status().download_rate);

			if(!sm->getBool(SetupManager::DISABLE_UPLOAD_SPEED))
				noteBookManager.setMaxUploadSpeed(sm->get(SetupManager::UPLOAD_SPEED) * 1024);
			else
				noteBookManager.setMaxUploadSpeed(ssm->status().upload_rate);

			if (guiSetupManager.getBool(GuiSetupManager::SHOW_GRAPHIC_STATISTICS))
				noteBookManager.updateGraphics();

			updateGraphics = guiSetupManager.get(GuiSetupManager::STATISTICS_TIME_UPDATE);
		}
		else
		updateGraphics--;
	}

	// Show log messages
	noteBookManager.flushLogMessages();

	// Show notification
	noteBookManager.flushNotification();

	return true;
}

void MainWindow::on_signal_hide()
{
	if (!systray.isEmbedded()){
		gtk_main_quit();
	}
	else{
		systray.setToggleShowButton(false);
	}
}

void MainWindow::signal_exit()
{
	if (systray.isEmbedded())
	{
		systray.setToggleShowButton(false);
		systray.hide();
	}
	else
		hide();

	gtk_main_quit();
}

void MainWindow::select_torrent_file_dialog()
{
	Glib::ustring torrentpathsave, torrentsavedest, torrentloaddir;
	boost::filesystem::path torrentpath;

	// Set Load directory
	if (guiSetupManager.get(GuiSetupManager::USE_LOAD_TORRENTS_DIRECTORY) &&
		!guiSetupManager.get(GuiSetupManager::ALLOW_CHANGE_LOAD_DIRECTORY))
	{
		torrentloaddir = guiSetupManager.get(GuiSetupManager::
												 LOAD_TORRENTS_DIRECTORY);
	}
	else if(guiSetupManager.get(GuiSetupManager::LAST_LOAD_DIRECTORY) != "")
		torrentloaddir = guiSetupManager.get(GuiSetupManager::LAST_LOAD_DIRECTORY);
	else
		torrentloaddir = getcwd(NULL, 0);

	// Set Save directory
	if (guiSetupManager.get(GuiSetupManager::USE_DEFAULT_FILE_DIRECTORY) &&
		!guiSetupManager.get(GuiSetupManager::DEFAULT_FILE_DIRECTORY).empty())
	{
		torrentpathsave = guiSetupManager.get(GuiSetupManager::DEFAULT_FILE_DIRECTORY);
	}
	else
		torrentpathsave = guiSetupManager.get(GuiSetupManager::LAST_SAVE_DIRECTORY);

	if (torrentpathsave.empty()) torrentpathsave = getcwd(NULL, 0);

	// Run the dialog
	std::vector<Utils::torrent_data> torrents_data;
	SetupManagerPtr sm = Core::get_setup_manager();
	LoadTorrentDialog loadTorrentDialog(sm->get(SetupManager::STORAGE_TYPE),
								torrentloaddir, torrentpathsave, &torrents_data
								);

	int result = loadTorrentDialog.run();
	loadTorrentDialog.hide();

	if (result == Gtk::RESPONSE_CANCEL || result == Gtk::RESPONSE_DELETE_EVENT)
		return;

	if (!torrents_data.empty())
	{
		torrentpath = torrents_data.back().torrentpath.c_str();

		// Update Directories
		guiSetupManager.set(GuiSetupManager::LAST_LOAD_DIRECTORY,
					 torrentpath.branch_path().string());

		guiSetupManager.set(GuiSetupManager::LAST_SAVE_DIRECTORY,
							torrents_data.back().savepath);

		sm->set(SetupManager::LAST_SAVE_DIRECTORY, torrents_data.back().savepath);
	}


	for (int i=0; i<torrents_data.size(); i++)
	{
		torrentpath = torrents_data[i].torrentpath.c_str();

		if (!boost::filesystem::exists(torrentpath))
		{
			Utils::warningDialog(_("Selected file does not exists"),
								 torrents_data[i].torrentpath);
			return;
		}

		torrentsavedest = Glib::build_filename(get_base_directory().native_directory_string()
											   , torrentpath.leaf());

		// Copy torrent to desired place only if desired place and load place are not the same place
		if (guiSetupManager.getBool(GuiSetupManager::USE_DOWNLOAD_TORRENTS_DIRECTORY)
			&& !guiSetupManager.get(GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY).empty())
		{
			if (torrentpath.branch_path() != guiSetupManager.get(GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY))
			{
				try
				{
				boost::filesystem::copy_file(torrentpath, boost::filesystem::path(
					Glib::build_filename(guiSetupManager.get(GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY)
					, torrentpath.leaf())));
				}
				catch(std::exception)
				{
				}

				if (guiSetupManager.get(GuiSetupManager::USE_REMOVE_ORIGINAL_TORRENT))
				{
					boost::filesystem::remove(torrentpath);
					torrentpath = Glib::build_filename(guiSetupManager.get(
						GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY), torrentpath.leaf());
				}
			}
		}

		SessionManagerPtr ssm;
		ssm = Core::get_session_manager();

		try
		{
			ssm->add_torrent_to_session(torrentpath, torrents_data[i].savepath,
								torrents_data[i].priorities, torrents_data[i].newfilenames, false);
		}
		catch(libtorrent::duplicate_torrent)
		{
			Utils::warningDialog(_("Torrent is already being downloaded"),
								 torrents_data[i].torrentpath);
			continue;
		}
		catch(queue_limit_reached)
		{
			Utils::warningDialog(_("The queue limit has been reached."),
								 torrents_data[i].torrentpath);
			return;
		}
	}
}

void MainWindow::updateGuiSetup(){
	std::ostringstream logmessage;
	libtorrent::session_settings settings;

	// Set the base directory
	if (!guiSetupManager.get(GuiSetupManager::DEFAULT_FILE_DIRECTORY).empty()){
			chdir(guiSetupManager.get(GuiSetupManager::DEFAULT_FILE_DIRECTORY).c_str());
		}

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_TORRENTSTAB))
		add_torrents_tab();

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_OPTIONSTAB))
		add_options_tab();

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_STATISTICSTAB))
		add_statistics_tab();

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_LOGTAB)){
		add_log_tab();
	}

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_SEARCHTAB)){
		add_search_tab();
	}

	if (guiSetupManager.getBool(GuiSetupManager::SHOW_TORRENTSTAB))
		noteBookManager.setActivePage(NoteBookManager::TORRENTS_TAB);

	// Tool bar
	menuBar.setShowToolBar(guiSetupManager.getBool(GuiSetupManager::SHOW_TOOLBAR));

	if (guiSetupManager.get(GuiSetupManager::USE_LOAD_TORRENTS_DIRECTORY) &&
		!guiSetupManager.get(GuiSetupManager::ALLOW_CHANGE_LOAD_DIRECTORY))
	{
			guiSetupManager.set(GuiSetupManager::LAST_LOAD_DIRECTORY,
								guiSetupManager.get(GuiSetupManager::LOAD_TORRENTS_DIRECTORY));
	}

	// StatisticsTab
	noteBookManager.showGraphStats(guiSetupManager.getBool(GuiSetupManager::SHOW_GRAPHIC_STATISTICS));

	// Log Tab
	noteBookManager.setLogTabActive(guiSetupManager.getBool(GuiSetupManager::USE_LOG_TAB));
	noteBookManager.setLogLevel(guiSetupManager.get(GuiSetupManager::LOG_LEVEL));
	noteBookManager.setLogColoredOutput(guiSetupManager.getBool(GuiSetupManager::LOG_COLORED_OUTPUT));
	noteBookManager.setLogAutoScroll(guiSetupManager.getBool(GuiSetupManager::LOG_AUTO_SCROLL));
	noteBookManager.setLogSaveToFile(guiSetupManager.getBool(GuiSetupManager::SAVE_LOG_TO_FILE));
}

bool MainWindow::getSelectedTorrentPosition(Lince::torrentPosition &tp, bool checkInfoTab)
{
	bool ret = false;
	if (noteBookManager.isTorrentsTabSelected())
	{
		tp = noteBookManager.getSelectedTorrentPosition();
		ret = true;
	}

	if (checkInfoTab && noteBookManager.isTorrentsInfoTabSelected())
	{
		tp = noteBookManager.getSelectedTorrentInfoTabPosition();
		ret = true;
	}

	return ret;
}

void MainWindow::stop_torrents(){
	if (noteBookManager.isTorrentsInfoTabSelected())
	{
		Core::get_torrents_manager()->get_torrent_at(
				noteBookManager.getSelectedTorrentInfoTabPosition())->stop();
		return;
	}

	std::vector<torrentPosition> tp = noteBookManager.getSelectedTorrentsPositions();
	for (int i=0; i < tp.size(); i++)
	{
		Core::get_torrents_manager()->get_torrent_at(tp[i])->stop();
	}
}


void MainWindow::resume_torrents(){
	if (noteBookManager.isTorrentsInfoTabSelected())
	{
		Core::get_torrents_manager()->get_torrent_at(
				noteBookManager.getSelectedTorrentInfoTabPosition())->start();
		return;
	}

	std::vector<torrentPosition> tp = noteBookManager.getSelectedTorrentsPositions();
	for (int i=0; i < tp.size(); i++)
	{
		Core::get_torrents_manager()->get_torrent_at(tp[i])->start();
	}
}

void MainWindow::start_all_torrents()
{
	Lince::Core::get_torrents_manager()->start_all_torrents();
}

void MainWindow::stop_all_torrents()
{
	Lince::Core::get_torrents_manager()->stop_all_torrents();
}

void MainWindow::remove_torrent(Torrent *t)
{
	bool remove_data=true;

	if(guiSetupManager.get(GuiSetupManager::CONFIRM_DATA_ELIMINATION)
	   && !t->is_complete())
	{
		int result;
		result = Utils::askDialogYesNoCancel(_("Remove Data?"),
											 _("Remove downloaded data"),
											 t->name());

		if (result == Gtk::RESPONSE_CANCEL ||result == Gtk::RESPONSE_DELETE_EVENT)
			return;
		else if (result == Gtk::RESPONSE_NO)
			remove_data = false;
		else
			remove_data = true;
	}

	if (guiSetupManager.get(GuiSetupManager::USE_DOWNLOAD_TORRENTS_DIRECTORY) &&
		guiSetupManager.get(GuiSetupManager::USE_REMOVE_TORRENT_WHEN_FINISHED))
	{
		boost::filesystem::remove(Glib::build_filename(guiSetupManager.get(
			GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY), t->file_name()));
	}

	noteBookManager.removeInfoTab(t->get_hash());

	if (!t->is_complete())
	{
		noteBookManager.removeDownloadingRowGivenTID(t->position());
		Core::get_session_manager()->remove_downloading_torrent(t->position(),
																remove_data);
		noteBookManager.updateDownloadingTorrents();
	}
	else
	{
		noteBookManager.removeDownloadedRowGivenTID(t->position());
		Core::get_session_manager()->remove_shared_torrent(t->position());
		noteBookManager.updateDownloadedTorrents();
	}
}

void MainWindow::remove_torrents(){
	Torrent * t;

	if (noteBookManager.isTorrentsInfoTabSelected())
	{
		t  = Core::get_torrents_manager()->get_torrent_at(
				noteBookManager.getSelectedTorrentInfoTabPosition());
		remove_torrent(t);
		return;
	}

	std::vector<torrentPosition> tp = noteBookManager.getSelectedTorrentsPositions();
	std::vector<torrentPosition>::const_reverse_iterator iter;

	std::sort(tp.begin(), tp.end());
	for (iter = tp.rbegin(); iter != tp.rend(); ++iter)
	{
		t = Core::get_torrents_manager()->get_torrent_at(*iter);
		remove_torrent(t);
	}
}

void MainWindow::recheck_torrent()
{
	torrentPosition position;

	if (!getSelectedTorrentPosition(position, false))
		return;

	Core::get_session_manager()->recheck_torrent(position);
}

void MainWindow::move_torrent_up(){
	int selected_position;
	bool remove_data=true;
	torrentPosition position;
	Torrent * t;

	if (!getSelectedTorrentPosition(position, false))
		return;

	if (position.first == 1)
		return;

	if (!position.second)
	{
		Core::get_torrents_manager()->move_downloading_torrent(position.first,
															   (position.first - 1));

		selected_position = noteBookManager.getDownloadingSelectedRow();

		if (!noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(position.first - 1);

		noteBookManager.updateDownloadingTorrents();

		if (noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(selected_position);
	}
	else{
		Core::get_torrents_manager()->move_shared_torrent(position.first,
														  (position.first - 1));

		selected_position = noteBookManager.getDownloadedSelectedRow();

		if (!noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(position.first - 1);

		noteBookManager.updateDownloadedTorrents();

		if (noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(selected_position);
	}
}

void MainWindow::move_torrent_down(){
	int selected_position, last_position;
	bool remove_data=true;
	torrentPosition position;
	Torrent * t;

	if (!getSelectedTorrentPosition(position, false))
		return;

	if (!position.second)
	{
		last_position = Core::get_torrents_manager()->downloading_torrents_count();

		if (position.first == last_position)
			return;

		Core::get_torrents_manager()->move_downloading_torrent(position.first,
															   (position.first + 1));

		selected_position = noteBookManager.getDownloadingSelectedRow();

		if (!noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(position.first + 1);

		noteBookManager.updateDownloadingTorrents();

		if (noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(selected_position);
	}
	else{
		last_position = Core::get_torrents_manager()->shared_torrents_count();

		if (position.first == last_position)
			return;

		Core::get_torrents_manager()->move_shared_torrent(position.first,
														  (position.first + 1));

		selected_position = noteBookManager.getDownloadedSelectedRow();

		if (!noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(position.first + 1);

		noteBookManager.updateDownloadedTorrents();

		if (noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(selected_position);
	}
}

void MainWindow::move_torrent_top(){
	int selected_position;
	bool remove_data=true;
	torrentPosition position;
	Torrent * t;

	if (!getSelectedTorrentPosition(position, false))
		return;

	if (position.first == 1)
		return;

	if (!position.second)
	{
		Core::get_torrents_manager()->move_downloading_torrent(position.first, 1);

		selected_position = noteBookManager.getDownloadingSelectedRow();

		if (!noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(1);

		noteBookManager.updateDownloadingTorrents();

		if (noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(selected_position);
	}
	else{
		Core::get_torrents_manager()->move_shared_torrent(position.first, 1);

		selected_position = noteBookManager.getDownloadedSelectedRow();

		if (!noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(1);

		noteBookManager.updateDownloadedTorrents();

		if (noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(selected_position);
	}
}

void MainWindow::move_torrent_bottom(){
	int selected_position, last_position;
	bool remove_data=true;
	torrentPosition position;
	Torrent * t;

	if (!getSelectedTorrentPosition(position, false))
		return;

	if (!position.second)
	{
		last_position = Core::get_torrents_manager()->downloading_torrents_count();

		selected_position = noteBookManager.getDownloadingSelectedRow();

		if (position.first == last_position)
			return;

		Core::get_torrents_manager()->move_downloading_torrent(position.first, last_position );

		if (!noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(last_position );

		noteBookManager.updateDownloadingTorrents();

		if (noteBookManager.isDownloadingSorted())
			noteBookManager.setSelectedDownloadingRow(selected_position);
	}
	else{
		last_position = Core::get_torrents_manager()->shared_torrents_count();

		if (position.first == last_position)
			return;

		Core::get_torrents_manager()->move_shared_torrent(position.first, last_position);

		selected_position = noteBookManager.getDownloadedSelectedRow();

		if (!noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(last_position);

		noteBookManager.updateDownloadedTorrents();

		if (noteBookManager.isDownloadedSorted())
			noteBookManager.setSelectedDownloadedRow(selected_position);
	}
}

void MainWindow::add_tinfo_tab(const Gtk::TreeModel::Path&, Gtk::TreeViewColumn*,
							   Glib::RefPtr<Gtk::ListStore>&)
{
	torrentPosition position;

	if (!getSelectedTorrentPosition(position, false))
		return;

	noteBookManager.addTorrentInfoTab(Core::get_torrents_manager()
									  ->get_torrent_at(position));
}

void MainWindow::add_tinfo_tab_from_menu(){
	torrentPosition position;

	if (!getSelectedTorrentPosition(position, false))
		return;

	noteBookManager.addTorrentInfoTab(Core::get_torrents_manager()
									  ->get_torrent_at(position));
}

void MainWindow::set_torrent_upload_speed(int value){
	int askedValue;
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	// Update Speed
	if (value == -1 || value){
		t->set_max_upload_speed(value);
	}
	else{
		if(Utils::askIntValueDialog(_("Enter an upload value"), _("Enter a value in kB/s to set torrent's upload speed"), askedValue)){
			if (askedValue <= 0 || askedValue > 20000) return;
			t->set_max_upload_speed(askedValue);
		}
	}
}

void MainWindow::set_torrent_download_speed(int value){
	int askedValue;
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	// Update Speed
	if (value == -1 || value){
		t->set_max_download_speed(value);
	}
	else{
		if(Utils::askIntValueDialog(_("Enter a download value"), _("Enter a value in kB/s to set torrent's download speed"), askedValue)){
			if (askedValue <= 0 || askedValue > 20000) return;
			t->set_max_download_speed(askedValue);
		}
	}
}

void MainWindow::set_torrent_max_connections(int value){
	int askedValue;
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	// Update Connections
	if (value){
		t->set_max_connects(value);
	}
	else{
		if(Utils::askIntValueDialog(_("Enter the connections value"), _("Enter a value in to set torrent's connections"), askedValue)){
			if (askedValue <= 0 || askedValue > 20000) return;
			t->set_max_connects(askedValue);
		}
	}
}

void MainWindow::set_torrent_upload_slots(int value){
	int askedValue;
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	// Update Slots
	if (value){
		t->set_upload_slots(value);
	}
	else{
		if(Utils::askIntValueDialog(_("Enter the upload slots value"), _("Enter a value to set torrent's upload slots"), askedValue)){
			if (askedValue <= 0 || askedValue > 20000) return;
			t->set_upload_slots(askedValue);
		}
	}
}

void MainWindow::set_torrent_share_ratio(double value){
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	// Update Ratio
	t->set_share_ratio(value);
}

void MainWindow::edit_torrent_trackers(){
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	EditTrackersDialog edt(t);
}

void MainWindow::torrent_force_reannounce(){
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	t->force_reannounce();
}

void MainWindow::export_torrent(){
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	std::string filename;
	boost::filesystem::basic_ofstream<char> out;

	filename = Utils::getFilenameFromSaveDialog(_("Save file as"), t->file_name(), "");

	Lince::save_entry(filename, t->create_torrent());
}

void MainWindow::set_upload_limit_from_systray(int value){
	int askedValue;

	// Update Speed
	if (!value){
		if(Utils::askIntValueDialog(_("Enter an upload value"), _("Enter a value in kB/s to set global upload speed"), askedValue)){
			if (askedValue <= 0 || askedValue > 20000) return;
			noteBookManager.setMaxConfigUploadSpeed(askedValue);
		}
	}
	else
		noteBookManager.setMaxConfigUploadSpeed(value);
}


void MainWindow::set_download_limit_from_systray(int value){
	int askedValue;

	// Download Speed
	if (!value){
		if(Utils::askIntValueDialog(_("Enter an upload value"), _("Enter a value in kB/s to set global upload speed"), askedValue)){
			if (askedValue <= 0 || askedValue > 20000) return;
			noteBookManager.setMaxConfigDownloadSpeed(askedValue);
		}
	}
	else
		noteBookManager.setMaxConfigDownloadSpeed(value);
}

void MainWindow::set_connections_limit_from_systray(int value)
{
	int askedValue;

	// Connections value
	if (!value){
		if(Utils::askIntValueDialog(_("Enter a value for connections"), _("Enter a value to set the global number of connections"), askedValue)){
			if (askedValue <= 0 || askedValue > 10000) return;
			noteBookManager.setMaxConfigConnects(askedValue);
		}
	}
	else
		noteBookManager.setMaxConfigConnects(value);
}

void MainWindow::set_upload_slots_limit_from_systray(int value)
{
	int askedValue;

	// Download Speed
	if (!value){
		if(Utils::askIntValueDialog(_("Enter a value for upload slots"), _("Enter a value to set the global number of upload slots"), askedValue)){
			if (askedValue <= 0 || askedValue > 10000) return;
			noteBookManager.setMaxConfigUploadSlots(askedValue);
		}
	}
	else
		noteBookManager.setMaxConfigUploadSlots(value);
}

void MainWindow::remove_current_tab(){
	noteBookManager.removeCurrentTab();
}

void MainWindow::make_torrent(){
	makeTorrentWindow.run();
}

void MainWindow::manage_window(){
	if (!isShown)
		show();
	else
		hide();
	isShown = !isShown;

}

void MainWindow::trackerLogin(){
	std::pair<Glib::ustring, Glib::ustring> loginData;
	torrentPosition position;
	Torrent *t;

	if (!getSelectedTorrentPosition(position, true))
		return;

	t = Core::get_torrents_manager()->get_torrent_at(position);

	if(Utils::askLoginDataDialog(_("Introduce log-in data"), loginData)){
		t->set_tracker_login(loginData.first, loginData.second);
	}
}

void MainWindow::show_config_options()
{
}
