/***************************************************************************
 *            XMLWriter.cc
 *
 *  Sun Jan  7 17:04:24 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
 
#include "XMLWriter.h"
 
 
XMLWriter::XMLWriter(){
 	LIBXML_TEST_VERSION
	
	writer   = NULL;
	formatfile = true;
}

XMLWriter::XMLWriter(const Glib::ustring & fname){
 	LIBXML_TEST_VERSION
	
	writer   = NULL;
	formatfile = true;
	filename = fname;
}

XMLWriter::~XMLWriter(){
	if (isWriterInitialized()){
		endDocument();
		freeWriter();
	}
	
	xmlCleanupParser();
}

bool XMLWriter::initializeWriter(){
	if (!filename.length()){
		return false;
	}
	
	writer = xmlNewTextWriterFilename(filename.c_str(), 0);
	
	if (writer) return true;
		
	return false;
}


/*!
 This is the first function that should be called to save some data into an xml
 file. Only utf8 encoding will be used and accepted as input data. By default 
 the file will be indented with tabs.
*/
bool XMLWriter::startDocument(const Glib::ustring &nodeName){
	int rc;
	
	if (!isWriterInitialized())
		initializeWriter();
	else
		return false;
	
	if (!isWriterInitialized())
		return false;
	
	if (formatfile){
		setFormatFile(true);
		xmlTextWriterSetIndentString(writer, BAD_CAST "\t");
	}
	
	encoding = "UTF-8";
	rc = xmlTextWriterStartDocument(writer, NULL, encoding.c_str(), NULL);
	if (rc < 0){
		return false;
	}
	
	return startElement(nodeName);
}


/*!
 Used to start an element in the xml file.
*/
bool XMLWriter::startElement(const Glib::ustring &nodeName){
	int rc;
		
	if (!isWriterInitialized() || !checkUtf8(nodeName))
		return false;
	
	rc = xmlTextWriterStartElement(writer, (xmlChar *) nodeName.c_str());
	
	if (rc < 0) {
        return false;
	}
	
	return true;
}

/*!
 Used to write a complete element to the file. It can't handle attributes.
*/
bool XMLWriter::writeElement(const Glib::ustring &elementname, const Glib::ustring &elementdata){
	int rc;
		
	if (!isWriterInitialized() || (!checkUtf8(elementname) || !checkUtf8(elementdata)))
		return false;
	
	rc = xmlTextWriterWriteElement(writer, (xmlChar *) elementname.c_str(), (xmlChar *) elementdata.c_str());
	
	if (rc < 0) {
        return false;
    }
    		
	return true;
}

/*!
 Used to add attributes to the current element.
*/
bool XMLWriter::addAttribute(const Glib::ustring &attr_name, const Glib::ustring &attr_data){
	int rc;
	
	if (!isWriterInitialized() || (!checkUtf8(attr_name) || !checkUtf8(attr_data)))
		return false;
	
	rc = xmlTextWriterWriteAttribute(writer, (xmlChar *) attr_name.c_str(), (xmlChar *) attr_data.c_str());
	
	if (rc < 0) {
        return false;
    }
	
	return true;
}

/*!
 Used to add data to the current element data must be a string.
*/
bool XMLWriter::addElementData(const Glib::ustring &elementdata){
	int rc;
		
	if (!isWriterInitialized() || !checkUtf8(elementdata))
		return false;
	
	rc = xmlTextWriterWriteFormatString(writer, "%s", elementdata.c_str());
	
	if (rc < 0) {
        return false;
    }
    		
	return true;
}

/*!
 Used to add data to the current element data must be an int.
*/
bool XMLWriter::addElementData(int elementdata){
	int rc;
	
	if (!isWriterInitialized())
		return false;
	
	rc = xmlTextWriterWriteFormatString(writer, "%i", elementdata);
	
	if (rc < 0) {
        return false;
    }
		
	return true;
}

/*!
 Used to add data to the current element data must be an float.
*/
bool XMLWriter::addElementData(float elementdata){
	int rc;
	
	if (!isWriterInitialized())
		return false;
	
	rc = xmlTextWriterWriteFormatString(writer, "%f", elementdata);
	
	if (rc < 0) {
        return false;
    }
		
	return true;
}

bool XMLWriter::addElementData(int64_t elementdata){
	int rc;
	
	if (!isWriterInitialized())
		return false;
	
	rc = xmlTextWriterWriteFormatString(writer, "%lli", elementdata);
	
	if (rc < 0) {
        return false;
    }
		
	return true;
}

/*!
 Used to add a comment. Can't be used after an endElement call.
*/
bool XMLWriter::addComment(const Glib::ustring &comment){
	int rc;
	
	if (!isWriterInitialized())
		return false;
	
	rc = xmlTextWriterWriteComment(writer, (xmlChar *) comment.c_str());
	
	if (rc < 0) {
        return false;
    }
    		
	return true;
}

/*!
 Used to end an element.
*/
bool XMLWriter::endElement(){
	int rc;
	
	if (!isWriterInitialized())
		return false;
	
	rc = xmlTextWriterEndElement(writer);
    if (rc < 0) {
        return false;
    }
	
	return true;
}

/*!
 Used to finish the xml file it will close all unclosed elements if it isn't
 called the destructor will do it.
*/
bool XMLWriter::endDocument(){
	int rc;
	
	if (!isWriterInitialized())
		return false;
	
	rc = xmlTextWriterEndDocument(writer);
    
	if (rc < 0){
		return false;
	}
	
	freeWriter();
	return true;
}

/*!
 Private function used to check that input data is utf8 compliant.
*/
bool XMLWriter::checkUtf8(const Glib::ustring & data){
	if (!xmlCheckUTF8((xmlChar *) data.c_str()))
		return false;
	
	return true;
}
